@@ -190,8 +190,7 @@ names into the internal module.
 
 =cut
 
-sub set_repo_names
-{
+sub set_repo_names {
 	($filter_name, $grip_name) = @_;
 }
 
@@ -202,8 +201,7 @@ module for use when calling reprepro.
 
 =cut
 
-sub set_noskip
-{
+sub set_noskip {
 	$noskip = shift;
 }
 
@@ -214,8 +212,7 @@ of executed.
 
 =cut
 
-sub set_dry_run
-{
+sub set_dry_run {
 	$dryrun = 1;
 }
 
@@ -226,20 +223,18 @@ C<Debian::Package::Compare::get_missing_sources> in a simple format
 
 =cut
 
-sub print_missing
-{
+sub print_missing {
 	my ($c, $s, $total);
 	my @lines=();
 	return undef unless (defined $filter_name and defined $grip_name);
 	my $missing = &get_missing_sources('unstable', "$filter_name", "$grip_name");
-	if (defined %$missing)
-	{
+	if (defined %$missing) {
 		$c = scalar (keys %$missing);
-		$s = ($c > 1) ? "sources" : "source";
-		print "INF: $c $s in the filter repository but not in Grip:\n";
+		$s = sprintf (ngettext("source", "sources", $c));
+		# Translators: INF is an abbreviation for 'information' for the user.
+		printf(_g("INF: %d %s in the filter repository but not in Grip:\n"), $c, $s);
 		$c = 0;
-		foreach my $pkg (sort keys %$missing)
-		{
+		foreach my $pkg (sort keys %$missing) {
 			$c++;
 			push @lines, "$c: $pkg\n\t(". $$missing{$pkg}{'source'}.")\n";
 		}
@@ -251,18 +246,15 @@ sub print_missing
 	return unless defined (%$missing);
 	@lines=();
 	$c = scalar (keys %$missing);
-	$s = ($c > 1) ? "binaries" : "binary";
-	print "INF: $c $s in the filter repository but not in Grip:\n";
+	$s = sprintf (ngettext("binary", "binaries", $c));
+	printf(_g("INF: %d %s in the filter repository but not in Grip:\n"), $c, $s);
 	$c = $total;
-	foreach my $pkg (sort keys %$missing)
-	{
+	foreach my $pkg (sort keys %$missing) {
 		my @alines=();
 		$c++;
 		my $line = "$c: $pkg\n\t";
-		foreach my $a (sort keys %{$$missing{$pkg}})
-		{
-			push @alines, "(". $$missing{$pkg}{$a}{"$filter_name"}.") ".
-				'{'.$a.'}'."\n";
+		foreach my $a (sort keys %{$$missing{$pkg}}) {
+			push @alines, "(". $$missing{$pkg}{$a}{"$filter_name"}.") ".'{'.$a.'}'."\n";
 		}
 		$line .= join ("\t", @alines);
 		push @lines, $line;
@@ -277,31 +269,28 @@ C<Debian::Package::Compare::get_missing_builddeps> in a simple format
 
 =cut
 
-sub print_build_deps
-{
+sub print_build_deps {
 	$base = &get_base;
-	my $host = `dpkg-architecture -qDEB_BUILD_ARCH_CPU`;
+	my $host = `LC_ALL=C dpkg-architecture -qDEB_BUILD_ARCH_CPU`;
 	chomp ($host);
 	my ($c, $s, $total);
 	my @lines=();
 	return undef unless (defined $filter_name and defined $grip_name);
 	my $missing = &get_missing_builddeps('unstable', "$filter_name", "$grip_name");
-	if (defined %$missing)
-	{
-		foreach my $p (sort keys %$missing)
-		{
+	if (defined %$missing) {
+		foreach my $p (sort keys %$missing) {
 			my @k = split (/ /, $p);
 			my $pkg = $k[0];
 			next if (not defined $pkg);
-			my $res = `apt-cache show $pkg 2>/dev/null`;
+			my $res = `LC_ALL=C apt-cache show $pkg 2>/dev/null`;
 			chomp ($res);
 			next if ($res ne "");
-			my $cache = `apt-cache showpkg $pkg 2>/dev/null`;
+			my $cache = `LC_ALL=C apt-cache showpkg $pkg 2>/dev/null`;
 			$cache =~ s/\n/ /g;
-			if ($cache !~ /Reverse Provides:/)
-			{
-				print "INF: Unable to check '$pkg'. It may be an virtual ".
-				"alternative package or might not be available on '$host'\n";
+			if ($cache !~ /Reverse Provides:/) {
+				printf(_g("INF: Unable to check '%s'. It may be an virtual ".
+				"alternative package or might not be available on '%s'\n"),
+					$pkg, $host);
 				next;
 			}
 			delete $$missing{$p};
@@ -311,8 +300,7 @@ sub print_build_deps
 			$match =~ s/ +$//;
 			my @pkgver = split (" ", $match);
 			my %pkgs = ();
-			foreach my $block (@pkgver)
-			{
+			foreach my $block (@pkgver) {
 				next if ($block =~ /^[0-9]/);
 				next if ($block =~ /^$/);
 				$pkgs{$block}++;
@@ -320,19 +308,16 @@ sub print_build_deps
 			my $rpkg = join (" | ", sort keys %pkgs);
 			next if ($rpkg eq "");
 			my @alternatives = ();
-			if (scalar keys %pkgs > 1)
-			{
+			if (scalar keys %pkgs > 1) {
 				@alternatives = sort keys %pkgs;
 				$rpkg = $alternatives[0];
 				my $end = scalar @alternatives;
-				if (scalar @alternatives > 2)
-				{
+				if (scalar @alternatives > 2) {
 					$end--;
 					print "INF: Alternatives for $rpkg: ";
 					print join (" | ", @alternatives[1..$end])."\n";
 				}
-				if (scalar @alternatives == 2)
-				{
+				if (scalar @alternatives == 2) {
 					$end--;
 					print "INF: Alternative for $rpkg: ";
 					print join (" ", $alternatives[$end])."\n";
@@ -341,22 +326,22 @@ sub print_build_deps
 			my $check = `reprepro -b ${base}grip list sid $rpkg`;
 			chomp $check;
 			$check =~ s/.*: //g;
-			if ($check ne "")
-			{
-				print "INF: '$rpkg' Provides '$pkg' and already exists ".
-					"in the Grip repository, skipping '$rpkg'.\n";
+			if ($check ne "") {
+				printf(_g("INF: '%s' Provides '%s' and already exists ".
+					"in the Grip repository, skipping '%s'.\n"),
+					$rpkg, $pkg, $rpkg);
 				next;
 			}
-			print "INF: Adding '$rpkg' to Provide: '$pkg'.\n";
+			printf(_g("INF: Adding '%s' to Provide: '%s'.\n"), $rpkg, $pkg);
 			$$missing{$rpkg} = \@alternatives;
 		}
 		$c = scalar (keys %$missing);
-		$s = ($c > 1) ? "build dependencies" : "build dependency";
-		print "INF: $c $s not yet in Grip:\n";
+		$s = sprintf (ngettext("dependency", "dependencies", $c));
+		printf(_g("INF: %s build %s not yet in Grip:\n"), $c, $s);
 		$c = 0;
 		print join (" ", sort keys %$missing)."\n";
 	}
-	print "INF: Done.\n";
+	printf (_g("INF: Done.\n"));
 }
 
 =head1 print_testing_status
@@ -373,34 +358,30 @@ the version in testing differs.
 
 =cut
 
-sub print_testing_status
-{
+sub print_testing_status {
 	my ($c, $plural);
 	return undef unless (defined $filter_name and defined $grip_name);
 	my $origu = &read_packages ('unstable', $filter_name);
 	my $origt = &read_packages ('testing', $filter_name);
 	my $britney = &get_britney_list("$filter_name", "$grip_name");
 	$c = scalar (keys %$britney);
-	if ($c > 0)
-	{
-		$plural = ($c > 1) ? "source packages need" : "source package needs";
-		print "INF: $c $plural to be migrated into testing.\n";
-		foreach my $brit (sort keys %$britney)
-		{
+	if ($c > 0) {
+		$plural = sprintf (ngettext("source package needs to be migrated",
+		"source packages need to be migrated", $c));
+		printf(_g("INF: %s %s into testing.\n"), $c, $plural);
+		foreach my $brit (sort keys %$britney) {
 			my $src = $$origu{$brit}{'Src'};
-			if ($src eq $brit)
-			{
-				print "INF: Source: $src Binary: $brit\n";
+			if ($src eq $brit) {
+				printf (_g("INF: Source: %s Binary: %s\n"), $src, $brit);
 			} else {
-				print "INF: Source: $src\n";
+				printf( _g("INF: Source: %s\n"), $src);
 			}
 			next if (not defined $$origu{$src}{'source'});
 			print "\tUnstable: ".$$origu{$src}{'source'}."\n";
 			print "\tTesting : ";
 			print $$origt{$src}{'source'} 
 				if (defined $$origt{$src}{'source'});
-			print "\n";
-			print "\tFilterU : ".$$origu{$src}{'source'}."\n";
+			print "\n\tFilterU : ".$$origu{$src}{'source'}."\n";
 			print "\tFilterT : ".$$origt{$src}{'source'}."\n";
 			print "\n\n";
 		}
@@ -434,8 +415,7 @@ more information.
 
 =cut
 
-sub grip_britney
-{
+sub grip_britney {
 	$base = &get_base;
 	return undef unless (-d $base);
 	my ($c, $plural, $str, $vers);
@@ -443,32 +423,27 @@ sub grip_britney
 	my $britney = &get_britney_list("$filter_name", "$grip_name");
 	my $complain = get_britney_complaint();
 	$c = scalar (keys %$britney);
-	if ($c > 0)
-	{
-		$plural = ($c > 1) ? "source packages need" : "source package needs";
-		print "INF: $c $plural to be migrated into testing.\n";
-		print "INF: ".join (' ', sort keys %$britney)."\n";
-		foreach my $brit (sort keys %$britney)
-		{
+	if ($c > 0) {
+		$plural = sprintf (ngettext("source package needs to be migrated",
+		"source packages need to be migrated", $c));
+		printf(_g("INF: %s %s into testing.\n"), $c, $plural);
+		print _g("INF: ").join (' ', sort keys %$britney)."\n";
+		foreach my $brit (sort keys %$britney) {
 			my $mode = 'copysrc';
 			$vers = $$britney{$brit}{'source'}{'unstable'};
 			# if no source version exists, this probably isn't a source
 			# package, so use copy, not copysrc
-			if (not defined $vers)
-			{
+			if (not defined $vers) {
 				$mode = 'copy';
-				print "INF: migrating $brit into testing.\n";
+				printf( _g("INF: migrating %s into testing.\n"), $brit);
+			} else {
+				# Translators: strings are package : version
+				printf( _g("INF: migrating %s %s into testing.\n"), $brit, $vers);
 			}
-			else
-			{
-				print "INF: migrating $brit $vers into testing.\n";
-			}
-			$str = "reprepro -b ${base}${grip_name} $mode testing".
-				" unstable $brit";
+			$str = "reprepro -b ${base}${grip_name} $mode testing unstable $brit";
 			(defined $dryrun) ? print "$str\n" : system ("$str 2>&1");
 			# will fail if $brit is not a source package.
-			$str = "reprepro -v -b ${base}locale $mode testing".
-				" unstable $brit";
+			$str = "reprepro -v -b ${base}locale $mode testing unstable $brit";
 			(defined $dryrun) ? print "$str\n" : system ("$str 2>&1");
 		}
 	}
@@ -491,14 +466,12 @@ is complete.
 
 =cut
 
-sub migrate_missing
-{
+sub migrate_missing {
 	my ($c, $s, $total);
 	my @lines=();
 	return undef unless (defined $filter_name and defined $grip_name);
 	my $missing = &get_missing_sources('testing', "$filter_name", "$grip_name");
-	if (defined %$missing)
-	{
+	if (defined %$missing) {
 		$c = scalar (keys %$missing);
 		my $debt  = &read_packages ('testing', $filter_name);
 		my $gripu  = &read_packages ('unstable', $grip_name);
@@ -507,33 +480,31 @@ sub migrate_missing
 		%gripunstable = %$gripu;
 		%griptesting = %$gript;
 		%debiantesting = %$debt;
-		$s = ($c > 1) ? "sources" : "source";
-		print "INF: $c $s in the filter repository but not in Grip:\n";
+		$s = sprintf (ngettext("source", "sources", $c));
+		printf( _g("INF: %s %s in the filter repository but not in Grip:\n"), $c, $s);
 		$c = 0;
-		foreach my $pkg (sort keys %$missing)
-		{
+		foreach my $pkg (sort keys %$missing) {
 			$c++;
-			if (not defined $griptesting{$pkg}{'source'})
-			{
+			if (not defined $griptesting{$pkg}{'source'}) {
 				# both sides of the dpkg --compare-versions must exist
 				# or we get lots of horrible output from dpkg.
 				if ((not defined $gripunstable{$pkg}{'source'}) or
-					(not defined $debiantesting{$pkg}{'source'}))
-				{
-					carp ("ERR: no data for $pkg in Grip unstable.\n");
+					(not defined $debiantesting{$pkg}{'source'})) {
+					my $msg = sprintf (_g("INF: Removing %s from filter testing.\n"), $pkg);
+					system ("reprepro -b ${base}${filter_name} -v removesrc $pkg");
+					carp ($msg);
 					next;
 				}
 				my $retval = system ("dpkg --compare-versions ".
 					$debiantesting{$pkg}{'source'}." '>>' ".
 					$gripunstable{$pkg}{'source'});
 				$retval /= 256;
-				if ($retval == 1)
-				{
-					my $str = "INF: $pkg is newer in Grip unstable (".
-					$gripunstable{$pkg}{'source'}.") than in Debian testing (".
-					$debiantesting{$pkg}{'source'}.")\n".
-					"Unable to migrate $pkg. Need to grip the package in".
-					" Debian testing.\n";
+				if ($retval == 1) {
+					my $str = sprintf(_g("INF: %s is newer in Grip unstable (%s)".
+						" than in Debian testing (%s)\n".
+						"Unable to migrate %s. Need to grip the package in".
+						" Debian testing.\n"), $pkg, $gripunstable{$pkg}{'source'},
+					$debiantesting{$pkg}{'source'}, $pkg);
 					warn ($str);
 					my $initial = convert_prefix ($pkg);
 					# source only supports main.
@@ -542,13 +513,12 @@ sub migrate_missing
 					# allow for epochs
 					$ver =~ s/^[0-9]://;
 					my $dsc = "${pkg}_${ver}.dsc";
-					print "Checking for $dsc\n";
-					opendir (DIR, "$dir")
-						or die ("Cannot open filter pool directory for $pkg\n");
+					printf( _g("Checking for %s\n"), $dsc);
+					my $msg = sprintf(_g("Cannot open filter pool directory for %s\n"), $pkg);
+					opendir (DIR, "$dir") or die ($msg);
 					my @dsc=grep(/\Q$dsc\E/, readdir (DIR));
 					closedir (DIR);
-					if ((exists $dsc[0]) and (-f "$dir/$dsc"))
-					{
+					if ((exists $dsc[0]) and (-f "$dir/$dsc")) {
 						grip_source ($pkg, $ver, 'testing', 'source');
 						&clean_incoming ($grip_name);
 					}
@@ -562,13 +532,11 @@ sub migrate_missing
 	return unless defined (%$missing);
 	@lines=();
 	$c = scalar (keys %$missing);
-	$s = ($c > 1) ? "binaries" : "binary";
-	print "INF: $c $s in the filter repository but not in Grip.\n";
+	$s = sprintf (ngettext("binary", "binaries", $c));
+	printf(_g("INF: %d %s in the filter repository but not in Grip.\n"), $c, $s);
 	$c = $total;
-	foreach my $pkg (sort keys %$missing)
-	{
-		foreach my $a (sort keys %{$$missing{$pkg}})
-		{
+	foreach my $pkg (sort keys %$missing) {
+		foreach my $a (sort keys %{$$missing{$pkg}}) {
 			# first get the source package name.
 			my $src= $debianunstable{$pkg}{'Src'};
 			# skip omitted packages
@@ -598,8 +566,7 @@ of the filter and grip repositories.
 
 =cut
 
-sub edos
-{
+sub edos {
 	$base = get_base();
 	my ($mode, $suite) = @_;
 	return undef unless (defined $filter_name and defined $grip_name);
@@ -609,11 +576,9 @@ sub edos
 	my @checks = ();
 	push @checks, $filter_name;
 	push @checks, $grip_name;
-	foreach my $rep (@checks)
-	{
+	foreach my $rep (@checks) {
 		print "\n";
-		foreach my $arch (@archlist)
-		{
+		foreach my $arch (@archlist) {
 			next if ($arch eq 'source');
 			# need to support components other than main.
 			my $str="edos-debcheck -explain -failures < ".
@@ -626,31 +591,28 @@ sub edos
 			$edos{$ret}{$arch}++;
 		}
 		my $c = scalar (keys %edos);
-		if ($c == 0)
-		{
-			print "INF: '$rep' appears well trimmed.\n";
+		if ($c == 0) {
+			printf( _g("INF: '%s' appears well trimmed.\n"), $rep);
 			exit 0 if ($mode eq 'edos');
 			return;
 		}
-		foreach my $detail (keys %edos)
-		{
+		foreach my $detail (keys %edos) {
 			print "\n";
-			print "INF: Missing dependencies in the '$rep' repository:\n";
-			print "INF: $detail\n";
-			print "INF: Affecting: " if (exists $edos{$detail});
-			foreach my $affected ($edos{$detail})
-			{
+			printf( _g("INF: Missing dependencies in the '%s' repository:\n"), $rep);
+			printf( _g("INF: %s\n"), $detail);
+			print _g("INF: Affecting: ") if (exists $edos{$detail});
+			foreach my $affected ($edos{$detail}) {
 				print join(' ', keys %$affected);
 			}
 			print "\n";
 		}
 		print "\n";
 	}
-	print "INF: Done\n\n";
+	print _g("INF: Done.\n");
+	print "\n";
 #	exit 0 if ($mode eq 'edos');
 	my @list=();
-	foreach my $pkg (sort keys %edos)
-	{
+	foreach my $pkg (sort keys %edos) {
 		$pkg =~ s/\(.*?\)//g;
 		$pkg =~ s/\|//g;
 		push @list, split (" ", $pkg);
@@ -658,8 +620,7 @@ sub edos
 	my %hash=();
 	# ensure the list only contains unique names
 	my $debu  = &read_packages ('unstable', $filter_name);
-	foreach my $l (@list)
-	{
+	foreach my $l (@list) {
 		# alternatives can sometimes cause duplicates to appear.
 		next if (exists $$debu{$l});
 		# special-case lsb because it brings in all of Qt
@@ -667,18 +628,16 @@ sub edos
 		$hash{$l}++;
 	}
 	$noskip='--noskipold' if (keys %hash);
-	if (not defined $dryrun)
-	{
+	if (not defined $dryrun) {
 		&extend_filter(join(" ", sort keys %hash));
 		&update_repo ();
-	}
-	else
-	{
-		print "INF: Need to add: ".join(" ", sort keys %hash)."\n";
-		foreach my $pkg (sort keys %hash)
-		{
+	} else {
+		print _g("INF: Need to add: ").join(" ", sort keys %hash)."\n";
+		foreach my $pkg (sort keys %hash) {
 			my $check = `reprepro -b ${base}filter list sid $pkg`;
-			print "ERR: '$pkg' exists in filter:\n$check\n"
+			# Translators: first is the package name, second is 
+			# output data from reprepro.
+			printf( _g("ERR: '%s' exists in filter:\n%s\n"), $pkg, $check)
 				if ($check ne "");
 		}
 	}
@@ -693,12 +652,12 @@ Call intermittently during long runs.
 
 =cut
 
-sub clean_incoming
-{
+sub clean_incoming {
 	$base = &get_base;
 	return undef unless (defined $grip_name);
-	print "INF: cleaning grip incoming directory\n";
-	print "INF: Stamp: " . `date`;
+	print _g("INF: cleaning grip incoming directory\n");
+	# Translators: timestamp.
+	print _g("INF: Stamp: ") . `date`;
 	# stage removals in case the argument list gets too long.
 	# probably do an opendir readdir instead.
 	unlink <${base}${grip_name}/incoming/*.changes>;
@@ -715,8 +674,7 @@ Only call before exit.
 
 =cut
 
-sub cleanup
-{
+sub cleanup {
 	&clean_incoming;
 	&update_filter;
 }
@@ -730,8 +688,7 @@ call only when necessary.
 
 =cut
 
-sub update_filter
-{
+sub update_filter {
 	return undef unless (defined $filter_name and defined $grip_name);
 	$base = &get_base;
 	my $debu  = &read_packages ('unstable', $filter_name);
@@ -739,13 +696,13 @@ sub update_filter
 	my $file = `mktemp -t "autogrip.XXXXXX"`;
 	chomp ($file);
 	open (PKGLIST, ">$file") or croak ("Unable to write pkglist");
-	foreach my $pkg (sort keys %$debu)
-	{
+	foreach my $pkg (sort keys %$debu) {
 		print PKGLIST "$pkg\t\t\t\tinstall\n";
 	}
 	close (PKGLIST);
 	# rename won't work on ant because of filesystem boundaries.
 	system ("mv $file ${base}${filter_name}/conf/pkglist");
+	chmod (0644, "${base}${filter_name}/conf/pkglist");
 }
 
 =head1 extend_filter
@@ -755,29 +712,26 @@ acceptable.
 
 =cut
 
-sub extend_filter
-{
+sub extend_filter {
 	$base = get_base;
 	return undef unless (defined $filter_name and defined $grip_name);
 	my $name = shift;
 	my @a = split(" ", $name);
-	print "INF: Adding $name\n";
+	printf( _g("INF: Adding %s\n"), $name);
 	# add the package to the pkglist from the arrays.
 	# names that don't exist will simply be ignored
 	my $file = `mktemp -t "autogrip.XXXXXX"`;
 	chomp ($file);
-	open (PKGLIST, ">$file") or croak ("Unable to extend pkglist.");
+	open (PKGLIST, ">$file") or croak (_g("Unable to extend pkglist."));
 	my $debu  = &read_packages ('unstable', $filter_name);
 	my @list = keys %$debu;
 	push @list, @a;
 	my %uniq=();
-	foreach my $u (@list)
-	{
+	foreach my $u (@list) {
 		$uniq{$u}++;
 	}
 	@list = sort keys %uniq;
-	foreach my $pkg (sort @list)
-	{
+	foreach my $pkg (sort @list) {
 		print PKGLIST "$pkg\t\t\t\tinstall\n";
 	}
 	close (PKGLIST);
@@ -792,31 +746,26 @@ to get updated packages.
 
 =cut
 
-sub update_repo
-{
+sub update_repo {
 	$base = get_base;
 	my ($c, $v);
 	my $verbose = shift;
 	$verbose = (defined $verbose) ? $verbose : 0;
 	$verbose = ($verbose > 0) ? $verbose : 0;
 	$v='';
-	for ($c = 0; $c <= $verbose; $c++)
-	{
+	for ($c = 0; $c <= $verbose; $c++) {
 		$v .= " -v";
 	}
-	print "INF: Updating filter repository\n";
+	print _g("INF: Updating filter repository\n");
 	if ((not defined $noskip) or ($noskip ne "--noskipold")) { $noskip = '' ; }
-	else { print "INF: $noskip set.\n"; }
-	if (not defined $dryrun)
-	{
+	else { printf( _g("INF: %s option is set.\n"), $noskip); }
+	if (not defined $dryrun) {
 		my $retval = system ("reprepro $v -b ${base}${filter_name} $noskip update 2>&1");
 		print "INF: Done\n";
 		# in case of error, reinitialise the pkglist file.
 		unlink "${base}${filter_name}/conf/pkglist" if ($retval);
-	}
-	else
-	{
-		print "INF: Dry run - no update done.\n";
+	} else {
+		print _g("INF: Dry run - no update done.\n");
 	}
 	&update_filter();
 }
@@ -835,8 +784,7 @@ not redirected to F</dev/null>.
 
 =cut
 
-sub incoming_locale
-{
+sub incoming_locale {
 	$base = get_base;
 	return undef unless (defined $grip_name);
 	my ($arch, $package, $version, $suite) = @_;
@@ -844,11 +792,10 @@ sub incoming_locale
 	my $l = &get_locale_roots ($suite, 'locale');
 	@locroots = sort @$l;
 	opendir (INCOMING, "${base}${grip_name}/incoming/")
-		or croak ("Unable to open ${base}${grip_name}/incoming/ : $!");
+		or croak (_g("Unable to open")." ${base}${grip_name}/incoming/ : $!");
 	@files=grep(/^${package}-locale-.*_${version}.*_.*\.tdeb$/, readdir (INCOMING));
 	closedir (INCOMING);
-	foreach $t (sort @files)
-	{
+	foreach $t (sort @files) {
 		# look for each component
 		next unless ($t =~ /${package}-locale-(.*)_${version}.*_.*\.tdeb$/);
 		next if (not defined $1);
@@ -857,24 +804,22 @@ sub incoming_locale
 		$suffix =~ s/-.*$//;
 		$suffix =~ s/\+.*$//;
 		$cmpnt = grep (/$suffix/, @locroots) ? $suffix : undef;
-		if (not defined $cmpnt)
-		{
-			carp "TDeb componentisation failed for $suffix:$t ";
-		}
-		else
-		{
+		if (not defined $cmpnt) {
+			# Translators: TDeb componentisation is a process of identifying
+			# the locale component from the translation package filename.
+			my $msg = sprintf(_g("TDeb componentisation failed for %s:%s "),
+				$suffix, $t);
+			carp ($msg);
+		} else {
 			$str = "reprepro --ignore=extension -C $cmpnt ".
 			"--export=never -b ${base}locale includedeb $suite ".
 			"${base}${grip_name}/incoming/$t";
-			(defined $dryrun) ? print "$str\n" :
-				system ("$str 2>&1");
+			(defined $dryrun) ? print "$str\n" : system ("$str 2>/dev/null");
 		}
 	}
-	if (scalar @files > 1)
-	{
+	if (scalar @files > 1) {
 		$str = "reprepro -v -b ${base}locale export $suite";
-		(defined $dryrun) ? print "$str\n" :
-			system ("$str 2>&1");
+		(defined $dryrun) ? print "$str\n" : system ("$str 2>/dev/null");
 	}
 }
 
@@ -885,8 +830,7 @@ hierarchy exist and create any that are missing.
 
 =cut
 
-sub check_dirs
-{
+sub check_dirs {
 	return undef unless (defined $filter_name and defined $grip_name);
 	mkdir ("$base/$filter_name")        if (not -d "$base/$filter_name");
 	mkdir ("$base/locale")              if (not -d "$base/locale");
@@ -922,8 +866,7 @@ bridge the security gap between Grip and Debian.
 
 =cut
 
-sub setup_repos
-{
+sub setup_repos {
 	$base = &get_base;
 	my ($mirror, $suite) = @_;
 	return undef unless (defined $filter_name and defined $grip_name);
@@ -939,47 +882,44 @@ sub setup_repos
 	wa wo xh yi zh zu / : @$l;
 	my $architectures = join (" ", @archlist) . " source";
 	my $cmpnts = "main " . join (" ", @locroots);
-	my $desc = "Emdebian $suite translation repository.\n";
-	my $label = "Emdebian-translation-$suite\n";
+	my $desc = sprintf(_g("Emdebian %s translation repository.\n"), $suite);
+	my $label = sprintf(_g("Emdebian-translation-%s\n"), $suite);
 	my $suite_name = ($suite eq "unstable") ? "sid" : "lenny";
 	my $udebcmpnts;
 	my $update;
 	my $override = "override.${suite_name}.extra.main";
 
-	print "INF: Initialising the reprepro directories\n";
-	if ((not -d "$base/$filter_name") and ($filter_name ne 'filter'))
-	{
-		print "The '${base}${filter_name}' directory does not exist.\n";
-		print "If this is the correct directory, please create it and\n";
-		print "ensure it is writable.\n";
-		croak ("Cannot write repository data: ${base}${filter_name}: $!");
-	}
-	else
-	{
+	# Translators: reprepro is the name of the command.
+	print _g("INF: Initialising the reprepro directories\n");
+	if ((not -d "$base/$filter_name") and ($filter_name ne 'filter')) {
+		printf( _g("The '%s' directory does not exist.\n".
+		"If this is the correct directory, please create it and\n".
+		"ensure it is writable.\n"), "${base}${filter_name}");
+		my $msg = sprintf (_g("Cannot write repository data:").
+			" ${base}${filter_name}: $!");
+		croak ($msg);
+	} else {
 		mkdir ("${base}${filter_name}");
 	}
-
-	if ((not -d "$base/$grip_name") and ($grip_name ne 'grip'))
-	{
-		print "The '${base}${grip_name}' directory does not exist.\n";
-		print "If this is the correct directory, please create it and\n";
-		print "ensure it is writable.\n";
-		croak ("Cannot write repository data: ${base}${grip_name}: $!");
-	}
-	else
-	{
+	if ((not -d "$base/$grip_name") and ($grip_name ne 'grip')) {
+		printf( _g("The '%s' directory does not exist.\n".
+			"If this is the correct directory, please create it and\n".
+			"ensure it is writable.\n"), "${base}${grip_name}");
+		my $msg = sprintf (_g("Cannot write repository data:").
+			" ${base}${grip_name}: $!");
+		croak ($msg);
+	} else {
 		mkdir ("${base}${grip_name}");
 	}
-
 	mkdir ("${base}locale") if (not -d "${base}locale");
 	mkdir ("${base}${filter_name}/conf") if (not -d "${base}${filter_name}/conf");
 	mkdir ("${base}${grip_name}/conf") if (not -d "${base}${grip_name}/conf");
 	mkdir ("${base}locale/conf") if (not -d "${base}locale/conf");
-
-	if (not -f "$base/locale/conf/distributions")
-	{
+	if (not -f "$base/locale/conf/distributions") {
+		# Translators, the string is a directory prefix.
+		my $msg = sprintf(_g("Cannot write %s/locale/conf/distributions"), $base);
 		open (DIST, ">$base/locale/conf/distributions")
-			or croak ("Cannot write $base/locale/conf/distributions :$!");
+			or croak ($msg." :$!");
 		print DIST "Origin: Debian\n";
 		print DIST "Label: $label\n";
 		print DIST "Suite: $suite\n";
@@ -992,13 +932,15 @@ sub setup_repos
 	# -dbg package => debug component
 	$cmpnts = "main dev doc debug";
 	$udebcmpnts = $cmpnts;
-	if (not -f "$base/$filter_name/conf/distributions")
-	{
+	if (not -f "$base/$filter_name/conf/distributions") {
 		$label = "DebianFilter";
 		$desc = "Debian unstable - filtered";
 		$update = "gripper1 udeb-unstable";
+		# Translators, the strings are a directory prefix.
+		my $msg = sprintf(_g("Cannot write %s/%s/conf/distributions"),
+			$base, $filter_name);
 		open (DIST, ">$base/$filter_name/conf/distributions")
-			or croak ("Cannot write $base/$filter_name/conf/distributions :$!");
+			or croak ($msg." :$!");
 		print DIST "Origin: Debian\n";
 		print DIST "Label: $label\n";
 		print DIST "Suite: $suite\n";
@@ -1010,13 +952,14 @@ sub setup_repos
 		print DIST "Update: $update\n";
 		close (DIST);
 	}
-	if (not -f "$base/$filter_name/conf/updates")
-	{
+	if (not -f "$base/$filter_name/conf/updates") {
 		$label = "DebianFilter";
 		$desc = "Debian unstable - filtered";
 		$update = "gripper1";
+		my $msg = sprintf(_g("Cannot write %s/%s/conf/updates"),
+			$base, $filter_name);
 		open (DIST, ">$base/$filter_name/conf/updates")
-			or croak ("Cannot write $base/$filter_name/conf/updates :$!");
+			or croak ($msg." :$!");
 		print DIST "Name: $update\n";
 		print DIST "Method: $mirror\n";
 		print DIST "Suite: $suite\n";
@@ -1032,12 +975,13 @@ sub setup_repos
 		print DIST "UDebComponents: $udebcmpnts\n";
 		close (DIST);
 	}
-	if (not -f "${base}${grip_name}/conf/distributions")
-	{
+	if (not -f "${base}${grip_name}/conf/distributions") {
 		$label = "EmdebianGrip";
 		$desc = "Emdebian Grip - unstable";
+		my $msg = sprintf(_g("Cannot write %s/%s/conf/distributions"),
+			$base, $grip_name);
 		open (DIST, ">${base}${grip_name}/conf/distributions")
-			or croak ("Cannot write ${base}${grip_name}/conf/distributions :$!");
+			or croak ($msg." :$!");
 		print DIST "Origin: Debian\n";
 		print DIST "Label: $label\n";
 		print DIST "Suite: $suite\n";
@@ -1049,15 +993,13 @@ sub setup_repos
 		print DIST "DebOverride: $override\n";
 		close (DIST);
 	}
-	if (not -f "${base}${grip_name}/conf/$override")
-	{
+	if (not -f "${base}${grip_name}/conf/$override") {
 		open (DIST, ">${base}${grip_name}/conf/$override")
 			or croak ("${base}${grip_name}/conf/$override : $!");
 		print DIST "\n";
 		close (DIST);
 	}
-	if (not -f "${base}${grip_name}/conf/incoming")
-	{
+	if (not -f "${base}${grip_name}/conf/incoming") {
 		open (DIST, ">$base/${grip_name}/conf/incoming")
 			or croak ("$base/${grip_name}/conf/incoming :$!");
 		print DIST "Name: autogrip\n";
@@ -1067,8 +1009,7 @@ sub setup_repos
 		close (DIST);
 	}
 	print "INF: Updating reprepro filter\n";
-	if (not -f "${base}${filter_name}/conf/pkglist")
-	{
+	if (not -f "${base}${filter_name}/conf/pkglist") {
 		open (PKGLIST, ">${base}${filter_name}/conf/pkglist")
 			or croak ("${base}${filter_name}/conf/pkglist :$!");
 		close (PKGLIST);
@@ -1076,7 +1017,7 @@ sub setup_repos
 	mkdir "${base}${grip_name}/incoming"
 		if (not -d "${base}${grip_name}/incoming");
 	# the filter always needs at least one package.
-	print "INF: Initialising the filter repository with a single package\n";
+	print _g("INF: Initialising the filter repository with a single package\n");
 	&extend_filter ('apt');
 	my $str = "reprepro -b ${base}${grip_name} export";
 	(defined $dryrun) ? print "$str\n" : system ($str);
@@ -1111,8 +1052,7 @@ under these rules.
 
 =cut
 
-sub convert_prefix
-{
+sub convert_prefix {
 	my $package = shift;
 	my ($index)=split(//, $package);
 	$index = substr($package, 0, 4) if ($package =~ /^lib/);
@@ -1132,63 +1072,65 @@ Once the source package has been processed, runs F<&clean_incoming>.
 
 =cut
 
-sub grip_source
-{
+sub grip_source {
 	my ($retval, $str);
 	$base = &get_base;
 	my ($package, $version, $suite, $arch) = @_;
 	return undef unless (defined $filter_name and defined $grip_name);
+	return undef unless (defined $package and $version);
 	# strip epochs only after finding the .dsc
 	$version =~ s/^[0-9]://;
 	my ($index) = convert_prefix($package);
-	# sources only need main.
+	my $clist = get_components ($suite, $filter_name);
 	my $dsc = "${base}${filter_name}/pool/main/$index/".
 		"$package/${package}_${version}.dsc";
-	croak ("Cannot read '$dsc' :$!") if (not -f "$dsc");
+	if (not -f "$dsc") {
+		foreach my $cmp (@$clist) {
+			$dsc = "${base}${filter_name}/pool/$cmp/$index/".
+				"$package/${package}_${version}.dsc";
+			last if (-f "$dsc");
+		}
+	}
+	croak (_g("Cannot read")." '$dsc' :$!") if (not -f "$dsc");
 	# copy the DSC over then grip the binaries.
 	my $dir = `mktemp -t -d "autogrip.XXXXXX"`;
 	chomp ($dir);
+	chdir ($dir);
 	# need to get the .debs from the repo, emgrip will then grip these.
 	# filter only uses main
 	# allow binNMUs etc. by adding .* to version
 	# this is source, so no $arch available.
-	chdir ($dir);
-	print "INF: getting source package: $dsc\n";
+	printf( _g("INF: getting source package: %s\n"), $dsc);
 	$str = "dget -ud file://${dsc}";
 	(defined $dryrun) ? print "$str\n" : system ($str);
 	$dsc = basename ($dsc);
 	my $a = get_archlist($suite, $grip_name);
 	@archlist = @$a;
-	# then run emgrip with a new .dsc mode
-	foreach $arch (@archlist)
-	{
-		next if ($arch eq 'source');
-		$str = "dpkg-architecture -a${arch} -c \"DEB_BUILD_OPTIONS='usegrip'".
-			" /usr/bin/emgrip -o $base/$grip_name/incoming $dsc 2>&1 \" 2>/dev/null";
-		(defined $dryrun) ? print "$str\n" : system ($str);
-	}
+	# then run emgrip with a new .dsc mode which only processes the src.
+	$str = "DEB_BUILD_OPTIONS='usegrip'".
+		" /usr/bin/emgrip -o $base/$grip_name/incoming $dsc";
+	(defined $dryrun) ? print "$str\n" : system ($str);
 	chdir ("../");
 	system ("rm -rf $dir");
 	# handle dsc's that contain no Section or Priority or component
 	my $single = get_single_package ("$suite", "$filter_name", "$package", "$arch");
-	if (not exists $$single{'Section'})
-	{
+	if (not exists $$single{'Section'}) {
 		$single = get_single_package ("unstable", "$filter_name", "$package", "$arch");
 	}
 	my $priority = (not exists $$single{'Priority'}) ?
 		"optional" : $$single{'Priority'};
-	print "INF: adding source package to grip using reprepro. ";
+	print _g("INF: adding source package to grip using reprepro. ");
 	print "Section: ".$$single{'Section'}." ";
 	print "Priority: ".$$single{'Priority'}."\n";
 	# all source packages go into main
-	$str = "reprepro -C main -V -S ".$$single{'Section'}." -P $priority".
+	$str = "reprepro -C main -S ".$$single{'Section'}." -P $priority".
 		" --ignore=brokensignatures -b ${base}${grip_name} includedsc $suite ".
 		"${base}${grip_name}/incoming/$dsc";
 	$retval = (defined $dryrun) ? print "$str\n" : system ("$str 2>&1");
 	&check_syscall ($retval);
 	# locale repo needs component to be main
 	print "INF: adding source package to locale using reprepro\n";
-	$str = "reprepro -C main -V -S ".$$single{'Section'}." -P ".$$single{'Priority'}.
+	$str = "reprepro -C main -S ".$$single{'Section'}." -P ".$$single{'Priority'}.
 		" --ignore=brokensignatures -b ${base}locale includedsc $suite ".
 		"${base}${grip_name}/incoming/$dsc";
 	$retval = (defined $dryrun) ? print "$str\n" : system ("$str 2>&1");
@@ -1223,20 +1165,24 @@ the Debian ftp-master overrides.
 
 =cut
 
-sub grip_binary
-{
+sub grip_binary {
 	my ($retval, $debbldopts, $outdir, $str);
 	my ($package, $version, $suite, $arch) = @_;
-	if (not defined $filter_name or not defined $grip_name)
-	{
-		my $msg = "filter name or grip name not defined!";
-		warn "Unable to proceed: '$msg'\n";
+	if (not defined $filter_name or not defined $grip_name) {
+		my $msg = _g("Unable to proceed: filter name or grip name not defined!");
+		warn "$msg\n";
 		return;
 	}
 	$base = &get_base;
 	# strip epochs
-	$version =~ s/^[0-9]://;
-	print "INF: Preparing to add single $package binary ($version) on $arch\n";
+	if (defined $version) {
+		$version =~ s/^[0-9]://;
+	} else {
+		return;
+	}
+	# Translators: package name, version, architecture.
+	printf( _g("INF: Preparing to add single %s binary (%s) on %s\n"),
+		$package, $version, $arch);
 	# identify the source package first.
 	# (use the requested suite in case of a package split)
 	# but handle -proposed-updates whereby the actual data is in
@@ -1244,21 +1190,21 @@ sub grip_binary
 	my $proposed = $suite;
 	$proposed =~ s/-proposed-updates$//;
 	my $single = get_single_package ($proposed, $grip_name, $package, $arch);
-	if (not exists $$single{'Section'})
-	{
-		$single = get_single_package ("unstable", "$filter_name", "$package", "$arch");
+	if (not exists $$single{'Section'}) {
+		$single = get_single_package ("$proposed", "$filter_name", "$package", "$arch");
 	}
-	if (not exists $$single{'Section'})
-	{
-		my $msg = "get_single_package (unstable, $filter_name, $package, $arch)";
-		warn "Unable to get results for '$msg'\n";
+	# allow for architectures pulled in from other mirrors.
+	# not ideal, it is slow to do it this way.
+	if (not exists $$single{'Section'}) {
+		# Translators, fields are: package, repo name, architecture.
+		printf(_g("Unable to find %s in %s for '%s', ignoring.\n"), $package, $filter_name, $arch);
 		return;
 	}
 	my $srcname = (defined $$single{'Source'}) ? $$single{'Source'} : $package;
 	$srcname =~ s/\(.*\)//;
 	chomp ($srcname);
 	$srcname =~ s/ //g;
-	print "INF: Using source package '$srcname'. " if ($srcname ne $package);
+	printf( _g("INF: Using source package '%s'. "), $srcname) if ($srcname ne $package);
 	print "Section: ".$single->{'Section'}." ";
 	print "Priority: ".$single->{'Priority'}."\n";
 	$single = get_single_package ($proposed, $filter_name, $package, $arch);
@@ -1266,21 +1212,20 @@ sub grip_binary
 	my @debs = ();
 	push @debs, "${base}$filter_name/$deb";
 	my $cmpnt = &switch_component ($$single{'Section'}, $package);
-	print "INF: Component: $cmpnt\n";
-	foreach my $d (@debs)
-	{
+	printf( _g("INF: Component: %s\n"), $cmpnt);
+	foreach my $d (@debs) {
 		next unless (-f "$d");
 		$str = "dpkg-architecture -a${arch} -c \"DEB_BUILD_OPTIONS='usegrip' ".
 			"/usr/bin/emgrip -o ${base}${grip_name}/incoming $d 2>&1 \" 2>/dev/null";
 		$retval = (defined $dryrun) ? print "$str\n" : system ($str);
 		&check_syscall($retval);
+		my $msg = _g("Cannot open");
 		opendir (INCOMING, "${base}${grip_name}/incoming/")
-			or croak ("Cannot open ${base}${grip_name}/incoming/ :$!");
+			or croak ("$msg ${base}${grip_name}/incoming/ :$!");
 		my @files=grep(/^\Q${package}_${version}\Eem._(${arch}|all)\.deb$/, readdir (INCOMING));
 		closedir (INCOMING);
-		foreach my $f (@files)
-		{
-			$str = "reprepro -V -C $cmpnt -b ${base}${grip_name} includedeb".
+		foreach my $f (@files) {
+			$str = "reprepro -v -C $cmpnt -b ${base}${grip_name} includedeb".
 				" $suite ${base}${grip_name}/incoming/$f";
 			$retval = (defined $dryrun) ? print "$str\n" : system ("$str 2>&1");
 			&check_syscall ($retval);
@@ -1299,23 +1244,28 @@ which sections.
 =cut
 
 sub switch_component {
+	# relocate a package merely by matching the binary package name
 	my @relocations = qw/ dev doc dbg /;
+	my %comps=();
 	my $cmpnt = "main";
 	my ($section, $pkg) = @_;
+	$cmpnt = $section if ($cmpnt ne $section);
+	# relocate a package by dint of the Section:
 	my $clist = get_components ('unstable', $grip_name);
+	foreach my $cmpnt (@$clist) {
+		$comps{$cmpnt}++;
+	}
 	# handle relocations - use main if fail.
+	$cmpnt = "main"  if (not exists($comps{$section}));
 	$cmpnt = "dev"   if ($section eq 'libdevel');
 	$cmpnt = "dev"   if ($section eq 'devel');
 	$cmpnt = "debug" if ($section eq 'debug');
 	$cmpnt = "doc"   if ($section eq 'doc');
 	$cmpnt = "main"  if (not defined $clist);
-	$cmpnt = "main"  if (scalar (grep(/$cmpnt/, @$clist)) == 0);
-	print "INF: Switching component: $cmpnt\n"
+	printf(_g("INF: Switching component: %s\n"), $cmpnt)
 		if ($cmpnt ne "main");
-	foreach my $reloc (@relocations)
-	{
-		if ($pkg =~ /\-${reloc}$/)
-		{
+	foreach my $reloc (@relocations) {
+		if ($pkg =~ /\-${reloc}$/) {
 			my $actual = $reloc;
 			$actual =~ s/^dbg$/debug/;
 			$cmpnt = "$actual";
@@ -1329,17 +1279,17 @@ sub scripts_version {
 	(defined $query) ? return $query : return "";
 }
 
-sub check_syscall
-{
+sub check_syscall {
 	my $signal = 0;
 	my $retval = shift;
 	return if (not defined $retval);
 	return if ($retval == 0);
 	$signal = $retval if ($retval < 256);
 	$retval /= 256 if ($retval >= 256);
-	print "INF: reprepro error: returned $retval\n" 
+	printf( _g("INF: reprepro error: returned %d\n"), $retval) 
 		if ($retval != 0 and $signal == 0);
-	croak ("ERR: reprepro died after receiving signal $signal\n")
+	my $msg = sprintf(_g("ERR: reprepro died after receiving signal %d\n"), $signal);
+	croak ($msg)
 		if ($signal > 0);
 }
 
