@@ -28,6 +28,8 @@ use strict;
 use warnings;
 use Data::Dumper;
 use File::Basename;
+use POSIX qw(locale_h);
+use Locale::gettext;
 use Emdebian::Grip;
 use Debian::Packages::Compare;
 
@@ -37,6 +39,8 @@ use vars qw/ $filter_name $grip_name $suite $base
 
 my $prog = basename($0);
 $our_version = &scripts_version();
+setlocale(LC_MESSAGES, "");
+textdomain("emdebian-grip");
 
 $mirror='http://ftp.uk.debian.org/debian'; # default
 $filter_name = 'filter';
@@ -77,23 +81,21 @@ while( @ARGV ) {
 		$grip_name = shift;
 	}
 	else {
-		die "$prog: Unknown option $_.\n";
+		die "$prog: "._g("Unknown option")." $_.\n";
 	}
 }
 
 $base .= '/' if ("$base" !~ m:/$:);
-if (not -d $base)
-{
-	print "ERR: Please specify an existing directory for the base-path: $base\n";
+if (not -d $base) {
+	printf (_g("ERR: Please specify an existing directory for the base-path: %s\n"), $base);
 	exit 1;
 }
 
-die ("ERR: no source package(s) specified.\n")
+die (_g("ERR: no source package(s) specified.\n"))
 	if ($mode eq "fail");
 
 # add any remaining arguments as source packages.
-while (scalar @ARGV > 0)
-{
+while (scalar @ARGV > 0) {
 	my $extra = shift;
 	push @cmd, $extra;
 }
@@ -101,12 +103,12 @@ while (scalar @ARGV > 0)
 &set_base($base);
 &set_repo_names ($filter_name, $grip_name);
 my $a = &get_archlist ($suite, $filter_name);
-die ("unable to get architecture list\n") if (not defined $a);
+die (_g("unable to get architecture list\n")) if (not defined $a);
 @archlist = @$a;
 my $l = &get_locale_roots ($suite, 'locale');
-die ("unable to get locale rootfs list\n") if (not defined $l);
+die (_g("unable to get locale rootfs list\n")) if (not defined $l);
 @locroots = @$l;
-die("no pkglist filter.\n")
+die(_g("no pkglist filter.\n"))
 	if ( not -f "${base}${filter_name}/conf/pkglist" );
 
 my $gript = &read_packages ($suite, $grip_name);
@@ -118,11 +120,9 @@ my %build=();
 my %migrate=();
 my %seen=();
 
-foreach my $query (sort @cmd)
-{
-	if (not defined $debian{$query}{'Src'})
-	{
-		print "INF: Cannot find $query in Debian $suite. Skipping it.\n";
+foreach my $query (sort @cmd) {
+	if (not defined $debian{$query}{'Src'}) {
+		printf (_g("INF: Cannot find %s in Debian %s. Skipping it.\n"), $query, $suite);
 		next;
 	}
 	my $src = (defined $debian{$query}{'Src'}) ?
@@ -132,9 +132,10 @@ foreach my $query (sort @cmd)
 	print "INF: Using $src as source package for '$query'.\n";
 	my $prefix = convert_prefix ($src);
 	my $ver = $debian{$src}{'source'};
-	if (not defined $ver)
-	{
-		warn ("undefined Debian version for $query in $suite");
+	if (not defined $ver) {
+		# Translators: first string is the package name, second the suite.
+		my $msg = sprintf (_g("undefined Debian version for %s in %s"), $query, $suite);
+		warn ($msg);
 		next;
 	}
 	next if ($query eq "acpi-support-base");
@@ -143,78 +144,63 @@ foreach my $query (sort @cmd)
 	# strip binNMU's
 	$ver =~ s/\+b[0-9]$//;
 	my $dsc = "${base}filter/pool/main/${prefix}/${src}/${src}_${ver}.dsc";
-	if (not -f $dsc)
-	{
-		warn "$dsc not found\n";
+	if (not -f $dsc) {
+		my $msg = sprintf (_g("%s not found\n"), $dsc);
+		warn ($msg);
 		next;
 	}
 	my $exists = $grip{$src}{'source'};
-	$exists =~ s/em[0-9]$//;
-	if ($ver eq $exists)
-	{
+	$exists =~ s/em[0-9]$// if (defined $exists);
+	if (defined $exists and ($ver eq $exists)) {
 		$migrate{$query}=$exists;
-		warn ("ERR: $src ($exists) exists in Grip $suite - skipping '$src'.\n");
-		next;
+		# Translators: strings are: source_package, version, suite, source_package.
+		my $msg = sprintf (_g("ERR: %s (%s) exists in Grip %s - skipping '%s'.\n"),
+			$src, $exists, $suite, $src);
 	}
-	print "INF: Checking $query ($exists).\n";
-	if (defined $grip{$query})
-	{
+	printf(_g("INF: Checking %s (%s).\n"), $query, $exists)
+		if (defined $exists);
+	if (defined $grip{$query}) {
 		my $exists = $grip{$query}{'source'};
-		if (defined $exists)
-		{
+		if (defined $exists) {
 			$exists =~ s/em[0-9]$//;
-			if ($exists eq $ver)
-			{
-				print "Skipping $query $ver : $exists\n";
+			if ($exists eq $ver) {
+				printf (_g("Skipping %s %s : %s\n"), $query, $ver, $exists);
 				next;
 			}
 		}
+	} else {
+		printf (_g("Adding %s %s\n"), $query, $ver);
 	}
-	else
-	{
-		print "Adding $query $ver\n";
-	}
-	if (defined $go)
-	{
+	if (defined $go) {
 		# subsequent changes are likely to repeat existing work.
 		&grip_source ($src, $ver, $suite, 'source')
 			unless (exists $seen{$src});
 		$seen{$src}++;
 		my $arch_hash = $debian{$query};
 		my @arch_list = keys %$arch_hash;
-		if ((scalar @arch_list == 1) and ($arch_list[0] eq 'source'))
-		{
+		if ((scalar @arch_list == 1) and ($arch_list[0] eq 'source')) {
 			&grip_binary($query, $ver, $suite, 'all');
-		}
-		else
-		{
-			foreach my $arch (sort @arch_list)
-			{
+		} else {
+			foreach my $arch (sort @arch_list) {
 				next if ($arch eq 'Src');
 				next if ($arch eq 'source');
 				&grip_binary ($query, $ver, $suite, $arch);
 			}
 		}
-	}
-	else
-	{
+	} else {
 		# subsequent changes are likely to repeat existing work.
-		print "INF: Would grip $src ($ver)\n"
+		printf(_g("INF: Would grip %s (%s)\n"), $src, $ver)
 			unless (exists $seen{$src});
 		$seen{$src}++;
 		my $arch_hash = $debian{$query};
 		my @arch_list = keys %$arch_hash;
-		if ((scalar @arch_list == 1) and ($arch_list[0] eq 'source'))
-		{
-			print "INF: Would grip $query ($ver) {all}\n";
-		}
-		else
-		{
-			foreach my $arch (sort @arch_list)
-			{
+		if ((scalar @arch_list == 1) and ($arch_list[0] eq 'source')) {
+			printf (_g("INF: Would grip %s (%s) {all}\n"), $query, $ver);
+		} else {
+			foreach my $arch (sort @arch_list) {
 				next if ($arch eq 'Src');
 				next if ($arch eq 'source');
-				print "INF: Would grip $query ($ver) {".$arch."})\n";
+				printf(_g("INF: Would grip %s (%s) {%s})\n"), $query, $ver, $arch);
 			}
 		}
 	}
@@ -224,7 +210,7 @@ exit 0;
 
 =head1 Copyright and Licence
 
- Copyright (C) 2009  Neil Williams <codehelp@debian.org>
+ Copyright (C) 2009-2010  Neil Williams <codehelp@debian.org>
 
  This package is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -243,12 +229,12 @@ exit 0;
 
 sub usageversion
 {
-	print(STDERR <<END)
-$prog - handle direct updates to testing
-version $our_version
+	printf STDERR (_g("
+%s - handle direct updates to testing
+version %s
 
-Syntax: $prog -b PATH [OPTIONS] -s|--source PACKAGES...
-        $prog -?|-h|--help|--version
+Syntax: %s -b PATH [OPTIONS] -s|--source PACKAGES...
+        %s -?|-h|--help|--version
 
 Commands:
 -b|--base-path PATH:           path to the top level grip directory [required]
@@ -276,9 +262,9 @@ that script, the suite name itself cannot be altered.
 
 Most of the work is done in Emdebian::Grip.
 
-$prog also updates the locale repository, shared by Emdebian
+%s also updates the locale repository, shared by Emdebian
 Grip and Emdebian Crush.
 
-END
-	or die "$0: failed to write usage: $!\n";
+"), $prog, $our_version, $prog, $prog, $prog)
+	or die ("$0: "._g("failed to write usage").": $!\n");
 }
