@@ -73,6 +73,7 @@ replicated to test pkglist files from remote repositories.
 
 my $prog = basename($0);
 $filter_name = 'filter';
+my $www;
 
 while( @ARGV ) {
 	$_= shift( @ARGV );
@@ -94,20 +95,33 @@ while( @ARGV ) {
 	elsif (/^(-f|--file)$/) {
 		$file = shift;
 	}
+	elsif (/^(-w|--www)$/) {
+		$www++;
+	}
 	else {
 		die "$prog: Unknown option $_.\n";
 	}
 }
 
 die "ERR: Please specify an existing directory for the base-path.\n"
-	if (not defined $base);
+	if (not defined $base and not defined $www);
 
 $base .= '/' if ("$base" !~ m:/$:);
-if (not -d $base)
+if (not -d $base and not defined $www)
 {
 	print "ERR: Please specify an existing directory for the base-path: $base";
 	exit 1;
 }
+if (defined $www)
+{
+	my $dir = `mktemp -t -d "grip-select.XXXXXX"`;
+	chomp ($dir);
+	mkdir "$dir/conf";
+	system ("wget -O $dir/conf/pkglist http://www.emdebian.org/grip/pkglist");
+	$www = $dir;
+	$base = $dir;
+	$filter_name = "/";
+}
 
 %pkg=();
 open (SELECT, "$file") or die ("ERR: No file: $file $!\n");
@@ -122,7 +136,7 @@ foreach $line (@lines)
 	$pkg{$line}++;
 }
 open (SELECT, "${base}${filter_name}/conf/pkglist") or
-	die ("ERR: No pkglist in ${base}${filter_name}conf/: $!\n");
+	die ("ERR: No pkglist in ${base}${filter_name}/conf/: $!\n");
 @filter=<SELECT>;
 close (SELECT);
 foreach $have (@filter)
@@ -135,6 +149,59 @@ foreach $have (@filter)
 print "Of ".scalar (@lines)." selected packages, ";
 print scalar (keys %pkg)." need to be added:\n";
 print join (" ", sort keys %pkg)."\n";
+print "\n";
+if (defined $www and -d $www)
+{
+	$base = "/var/lib/apt";
+	$filter_name = "";
+}
+if (-d "${base}${filter_name}/lists/")
+{
+	my %outside=();
+	my %debian=();
+	print "${base}${filter_name}/lists/\n";
+	my $arch = `dpkg-architecture -qDEB_BUILD_ARCH_CPU`;
+	chomp ($arch);
+	my $list = `find "${base}${filter_name}/lists/" -name '*sid_main*Packages' -type f -ctime -7|grep -v uPackages|grep -m1 $arch`;
+	chomp ($list);
+	if ($list eq "")
+	{
+		print "failed with sid\n";
+		$list = `find "${base}${filter_name}/lists/" -name '*unstable_main*Packages' -type f|grep -m1 $arch`;
+		chomp ($list);
+		print "list=:$list:\n";
+	}
+	if (-f "$list")
+	{
+		print "Checking $list\n";
+		foreach my $p (sort keys %pkg)
+		{
+			print "Checking $p: ";
+			my $avail = `grep-available -PX $p $list|grep -m1 Package|cut -d':' -f2`;
+			chomp ($avail);
+			if ($avail eq "")
+			{
+				print "Not in Debian.\n";
+				$outside{$p}++;
+			}
+			else
+			{
+				print "Needs to be added to Emdebian.\n";
+				$debian{$p}++;
+			}
+		}
+		print scalar (keys %outside)." packages are not in Debian:\n";
+		print join (" ", sort keys %outside)."\n";
+		print scalar (keys %debian)." need to be added to Emdebian Grip:\n";
+		print join (" ", sort keys %debian)."\n";
+	}
+}
+
+if (defined $www and -d $www)
+{
+	system ("rm -rf $www");
+}
+exit 0;
 
 sub usageversion
 {
@@ -145,8 +212,9 @@ Syntax: $prog -b PATH -f FILE [--filter-name STRING]
         $prog -?|-h|--help|--version
 
 Commands:
--b|--base-path PATH:        path to the top level grip directory [required]
+-b|--base-path PATH:        path to the top level grip directory
 -f|--file FILE:             filename containing the selections [required]
+-w|--www:                   use the current www.emdebian.org filter instead.
 
 -?|-h|--help|--version:     print this help message and exit
 
