@@ -236,7 +236,7 @@ foreach my $pkg (@srcset) {
 	if (defined $debianstable{$pkg}{'source'}) {
 		push @output, "DebianVersion: $debianstable{$pkg}{'source'}";
 	} else {
-		$errors{$pkg} .= _g("Error: missing source package in stable.");
+		$errors{$pkg} .= _g("Error: source package is not in Debian stable.");
 		next;
 	}
 	if (defined $gripstable{$pkg}{'source'}) {
